package
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	
	import funnel.Configuration;
	import funnel.Fio;
	import funnel.FunnelEvent;
	import funnel.OUT;
	import funnel.Scaler;

	public class FunnelManager extends Sprite
	{
		private var fio:Fio;

		private var scaler:Scaler;
		private var scaler2:Scaler;
		private var value0:int=3;
		private var value2:int=3;
		private var state:int=0;

		public var onReceive:Function;

		public static const NOT_DETERMINED:int=0;
		public static const PAPER:int=1;
		public static const SCISSORS:int=2;
		public static const ROCK:int=3;
		
		// 要チューニング
		private static const INDEX_FINGER_OPEN:Number = 0.4
		private static const INDEX_FINGER_CLOSE:Number = 0.27;
		private static const RING_FINGER_OPEN:Number = 0.48;
		private static const RING_FINGER_CLOSE:Number = 0.39;

		public function FunnelManager()
		{
			// FIOのFirmata用標準コンフィギュレーションを選択
			var config:Configuration=Fio.FIRMATA;
			// 通信トラフィックを最小限にするためにA0(人差し指)とA2(薬指)以外を出力に設定する
			config.setDigitalPinMode(13, OUT); // D13
			config.setDigitalPinMode(15, OUT); // A1
			config.setDigitalPinMode(17, OUT); // A3
			config.setDigitalPinMode(18, OUT); // A4
			config.setDigitalPinMode(19, OUT); // A5
			config.setDigitalPinMode(20, OUT); // A6
			config.setDigitalPinMode(21, OUT); // A7
			// Fioのインスタンスを生成
			// 引数：FIOのID、コンフィギュレーション、アドレス、ポート番号、サンプリング間隔
			fio=new Fio([1], config, "localhost", 9000, 12);

			// イベントリスナを追加
			fio.addEventListener(IOErrorEvent.IO_ERROR, trace);
			fio.addEventListener(FunnelEvent.READY, onReady);

			// スケーラーの初期化
			scaler=new Scaler(INDEX_FINGER_OPEN, INDEX_FINGER_CLOSE, 0, 100);
			scaler2=new Scaler(RING_FINGER_OPEN, RING_FINGER_CLOSE, 0, 100);
		}

		/**
		 * I/Oモジュールの初期化が完了したときの処理
		 */
		private function onReady(e:Event):void
		{
			trace(this, "onReady");
			addEventListener(Event.ENTER_FRAME, loop);
		}

		/**
		 * 毎フレーム行う処理
		 */
		private function loop(event:Event):void
		{
			// 人差し指の値をスケーリングする
			value0=scaler.processSample(fio.ioModule(1).analogPin(0).value);
			// 薬指の値をスケーリングする
			value2=scaler2.processSample(fio.ioModule(1).analogPin(2).value);

			// グーを認識
			if (value0 > 60 && value2 > 60 && state != ROCK)
			{
				trace(this, "グー");
				state=ROCK;
			}
			// チョキを認識
			else if (value0 < 20 && value2 > 60 && state != SCISSORS)
			{
				trace(this, "チョキ");
				state=SCISSORS;
			}
			// パーを認識
			else if (value0 < 20 && value2 < 20 && state != PAPER)
			{
				trace(this, "パー");
				state=PAPER;
			}

			if (onReceive != null)
				onReceive(state);

		}

	}
}